#!/usr/bin/env python3
"""Colors used across themes."""

LIGHT_COLORS = [
    "#1846b3",
    "#355e00",
    "#bb8600",
    "#dd2800",
    "#c510d2",
    "#9470fe",
    "#007dc2",
    "#00aeb3",
    "#008a47",
    "#942e00",
    "#f3398e",
    "#9e0077",
]

DARK_COLORS = [
    "#00a0ec",
    "#00bc70",
    "#deca00",
    "#ff7300",
    "#d83990",
    "#7555d3",
    "#8ac7ff",
    "#00f0ff",
    "#387200",
    "#aa2e00",
    "#ff7dc6",
    "#e960ff",
]


# the neutrals are from https://elastic.github.io/eui/#/theming/colors
# the naming is from a light-scheme perspective

LIGHT_SHADES = [
    # empty shade: primary background
    "#FFFFFF",
    # lightest shade: secondary background
    "#F1F4FA",
    # light shade: borders and dividers
    "#D3DAE6",
    # medium shade: subdued text
    "#98A2B3",
    # dark shade: secondary foreground
    "#69707D",
    # darkest shade: primary foreground, text
    "#343741",
]

DARK_SHADES = [
    # empty shade: primary background
    "#1D1E24",
    # lightest shade: secondary background
    "#25262E",
    # light shade: borders and dividers
    "#343741",
    # medium shade: subdued text
    "#535966",
    # dark shade: secondary foreground
    "#98A2B3",
    # darkest shade: primary foreground, text
    "#D4DAE5",
]
